#ifndef YSGLSLDRAWFONTBITMAP_IS_INCLUDED
#define YSGLSLDRAWFONTBITMAP_IS_INCLUDED
/* { */

#ifdef __cplusplus
extern "C" {
#endif

/* This enum is used in 2D drawing only. */
enum YSGLSL_BMPFONT_VIEWPORT_ORIGIN {
	YSGLSL_BMPFONT_TOPLEFT_AS_ORIGIN,
	YSGLSL_BMPFONT_BOTTOMLEFT_AS_ORIGIN
};

/* This enum is used in 2D or 3D drawing. */
enum YSGLSL_BMPFONT_FIRST_LINE_ALIGNMENT {
	YSGLSL_BMPFONT_ALIGN_TOPLEFT,
	YSGLSL_BMPFONT_ALIGN_BOTTOMLEFT
};

struct YsGLSLFontBitmapRenderer;

/*! This function creates a font bitmap renderer and returns a pointer to the new renderer. */
struct YsGLSLBitmapFontRenderer *YsGLSLCreateBitmapFontRenderer(void);

/*! This function deletes a font bitmap renderer. */
void YsGLSLDeleteBitmapFontRenderer(struct YsGLSLBitmapFontRenderer *renderer);

/*! This function activates the font bitmap renderer and returns an ID of the previously active program.. */
GLuint YsGLSLUseBitmapFontRenderer(struct YsGLSLBitmapFontRenderer *renderer);

/*! This function de-activates the font bitmap renderer. */
void YsGLSLEndUseBitmapFontRenderer(struct YsGLSLBitmapFontRenderer *renderer);

/*! This function specifies the orientation of the viewport.  The parameter viewportOrigin can be:
	YSGLSL_BMPFONT_TOPLEFT_AS_ORIGIN, or
	YSGLSL_BMPFONT_BOTTOMLEFT_AS_ORIGIN.  
	The renderer must be activated by YsGLSLUseBitmapFontRenderer before this function. */
void YsGLSLSetBitmapFontRendererViewportOrigin(
	struct YsGLSLBitmapFontRenderer *renderer,enum YSGLSL_BMPFONT_VIEWPORT_ORIGIN viewportOrigin);

/*! This function specifies the alignment of the bitmap fonts.  The parameter firstLineAlignment can be:
	YSGLSL_BMPFONT_ALIGN_TOPLEFT, or
	YSGLSL_BMPFONT_ALIGN_BOTTOMLEFT. 
	The renderer must be activated by YsGLSLUseBitmapFontRenderer before this function. */
void YsGLSLSetBitmapFontRendererFirstLineAlignment(
	struct YsGLSLBitmapFontRenderer *renderer,enum YSGLSL_BMPFONT_FIRST_LINE_ALIGNMENT firstLineAlignment);

/*! This function requests the bitmap font size that is close to wid*hei pixels. */
void YsGLSLBitmapFontRendererRequestFontSize(struct YsGLSLBitmapFontRenderer *renderer,int wid,int hei);

/*! This function returns the font width in pixels of the font renderer. 
    This function can be used after YsGLSLBitmapFontRendererRequestFontSize to check what font size is actually chosen. */
int YsGLSLGetBitmapFontRendererFontWidth(struct YsGLSLBitmapFontRenderer *renderer);

/*! This function returns the font width in pixels of the font renderer. 
    This function can be used after YsGLSLBitmapFontRendererRequestFontSize to check what font size is actually chosen. */
int YsGLSLGetBitmapFontRendererFontHeight(struct YsGLSLBitmapFontRenderer *renderer);
	
/*! This function specifies the color of the fonts.  The renderer must be activated by YsGLSLUseBitmapFontRenderer before this function. */
void YsGLSLSetBitmapFontRendererColor4ub(struct YsGLSLBitmapFontRenderer *renderer,unsigned char r,unsigned char g,unsigned char b,unsigned char a);

/*! This function specifies the color of the fonts.  The renderer must be activated by YsGLSLUseBitmapFontRenderer before this function. */
void YsGLSLSetBitmapFontRendererColor3ub(struct YsGLSLBitmapFontRenderer *renderer,unsigned char r,unsigned char g,unsigned char b);

/*! This function specifies the color of the fonts.  The renderer must be activated by YsGLSLUseBitmapFontRenderer before this function. */
void YsGLSLSetBitmapFontRendererColor4f(struct YsGLSLBitmapFontRenderer *renderer,float r,float g,float b,float a);

/*! This function specifies the color of the fonts.  The renderer must be activated by YsGLSLUseBitmapFontRenderer before this function. */
void YsGLSLSetBitmapFontRendererColor3f(struct YsGLSLBitmapFontRenderer *renderer,float r,float g,float b);

/*! This function sets the modelView matrix.  ModelView and projection matrices are used when bitmap font is drawn in a 3D scene. 
    The renderer must be activated by YsGLSLUseBitmapFontRenderer before this function. */
void YsGLSLSetBitmapFontRendererModelViewfv(struct YsGLSLBitmapFontRenderer *renderer,const GLfloat modelViewMat[16]);

/*! This function sets the projection matrix.  ModelView and projection matrices are used when bitmap font is drawn in a 3D scene. 
    The renderer must be activated by YsGLSLUseBitmapFontRenderer before this function. */
void YsGLSLSetBitmapFontRendererProjectionfv(struct YsGLSLBitmapFontRenderer *renderer,const GLfloat projectionMat[16]);

/*! This function sets viewport size. 
    The renderer must be activated by YsGLSLUseBitmapFontRenderer before this function. */
void YsGLSLSetBitmapFontRendererViewportSize(struct YsGLSLBitmapFontRenderer *renderer,int width,int height);

/*! This function draws str at screen coord of (x,y). 
    After this function, active texture will be set to GL_TEXTURE0, and the binding of GL_TEXTURE2D will change. */
void YsGLSLRenderBitmapFontString2D(struct YsGLSLBitmapFontRenderer *renderer,int x,int y,const char str[]);

/*! This function draws str at 3D coord of (x,y,z). 
    After this function, active texture will be set to GL_TEXTURE0, and the binding of GL_TEXTURE2D will change. */
void YsGLSLRenderBitmapFontString3D(struct YsGLSLBitmapFontRenderer *renderer,GLfloat x,GLfloat y,GLfloat z,const char str[]);



#ifdef __cplusplus
} // Closing extern "C"
#endif

/* } */
#endif
