#ifndef YSFONTRENDERER_IS_INCLUDED
#define YSFONTRENDERER_IS_INCLUDED
/* { */

#include <ysclass.h>
#include <ysbitmap.h>

class YsFontRenderer
{
public:
	/*! Utility function for counting width and height of a wide-char string.  It takes
	    new-line ('\n') into account and calculates the maximum width and height, which 
	    are returned in nWidth and nLine.
	    */
	template <class CHARTYPE>
	static void CountStringDimension(int &nWidth,int &nLine,const CHARTYPE str[]);

	/*! A convenience function for rendring an ASCII string.  It internally uses wide-char version of 
	    RenderString function. */
	YSRESULT RenderString(YsBitmap &bmp,const char str[],const YsColor &fgCol,const YsColor &bgCol) const;

	/*! Request a font by height in the number of pixels. */
	virtual YSRESULT RequestDefaultFontWithPixelHeight(unsigned int heightInPix)=0;

	/*! Render a wide-char string into a bitmap. */
	virtual YSRESULT RenderString(YsBitmap &bmp,const wchar_t wStr[],const YsColor &fgCol,const YsColor &bgCol) const=0;

	/*! Returns a tight rectangle dimension for the given wide-char string. */
	virtual YSRESULT GetTightRenderSize(int &wid,int &hei,const wchar_t wStr[]) const=0;
};

template <class CHARTYPE>
void YsFontRenderer::CountStringDimension(int &nWidth,int &nLine,const CHARTYPE str[])
{
	if(0!=str[0])
	{
		nLine=1;
		nWidth=0;

		int i,nCurWidth=0;
		for(i=0; 0!=str[i]; i++)
		{
			if('\n'==str[i])
			{
				nLine++;
				nCurWidth=0;
			}
			else
			{
				nCurWidth++;
				if(nWidth<nCurWidth)
				{
					nWidth=nCurWidth;
				}
			}
		}
	}
	else
	{
		nLine=0;
		nWidth=0;
	}
}


#include "ysfixedfontrenderer.h"
#include "yssystemfontrenderer.h"

/* } */
#endif
