/* ////////////////////////////////////////////////////////////

  File Name: edit_create_airfoil.h
  Copyright (c) 2014 Soji Yamakawa.  All rights reserved.
  http://www.ysflight.com

//////////////////////////////////////////////////////////// */

#ifndef EDIT_CREATE_AIRFOIL_IS_INCLUDED
#define EDIT_CREATE_AIRFOIL_IS_INCLUDED
/* { */

#include <fsgui.h>
#include <ysglbuffer.h>

class YsCreateAirfoilDialog : public FsGuiDialog
{
private:
	enum
	{
		NACA,
		FLAT,
		LENS,
		DIAMOND
	};

	FsGui3DMainCanvas *canvas;

	FsGuiButton *closeBtn,*createBtn,*helpBtn;
	FsGuiTabControl *airfoilTypeTab;

	FsGuiButton *thirdPointIsNormal,*thirdPointIsTangent;

	// NACA TAB >>
	int nacaTabIdent;
	FsGuiTextBox *nacaNumberTxt;
	FsGuiTextBox *nacaNumDivisionTxt;
	FsGuiButton *nacaSharpTailBtn;
	// NACA TAB <<

	// NACA6 TAB >>
	int naca6TabIdent;
	FsGuiDropList *naca6LabelDrp;
	FsGuiDropList *naca6MeanLineDrp;
	FsGuiButton *naca6ModifiedMeanLineBtn;
	FsGuiTextBox *naca6NumDivisionTxt;
	// NACA6 TAB <<

	// Supercritical TAB >>
	int supercriticalTabIdent;
	FsGuiTextBox *supercriticalNumDivisionTxt;
	FsGuiDropList *supercriticalLabelDrp;
	FsGuiButton *superCriticalMatchTailBtn;
	// Supercritical TAB <<

	// Joukowski TAB >>
	int JoukowskiTabIdent;
	FsGuiTextBox *JoukowskiATxt;
	FsGuiTextBox *JoukowskiDTxt;
	FsGuiTextBox *JoukowskiETxt;
	FsGuiTextBox *JoukowskiNumDivisionTxt;
	// Joukowski TAB <<

	// Clark-Y >>
	int ClarkYTabIdent;
	FsGuiTextBox *ClarkYNumDivisionTxt;
	// Clark-Y <<

	// Lens >>
	int lensTabIdent;
	FsGuiTextBox *lensNumDivisionTxt;
	FsGuiTextBox *lensH[2],*lensB[2];
	// Lens <<

public:
	YsVec3 front,back,thirdPoint;
	unsigned int frontVtKey,backVtKey;
	YsGLVertexBuffer airfoilVtxBuf;

private:
	YsCreateAirfoilDialog();
	~YsCreateAirfoilDialog();

public:
	void RecalculateAirfoil(void);
	void GetAirfoil(YsArray <YsVec3> &point) const;
	void GetAirfoil(YsArray <YsVec3> &point,const class Airfoil &airfoil) const;
	void GetUV(YsVec3 &u,YsVec3 &v) const;

	static YsCreateAirfoilDialog *Create(FsGui3DMainCanvas *canvas);
	static void Delete(YsCreateAirfoilDialog *toDel);

	void Make(void);

	virtual void OnButtonClick(FsGuiButton *btn);
	virtual void OnTextBoxChange(FsGuiTextBox *txt);
	virtual void OnDropListSelChange(FsGuiDropList *drp,int prevSel);
	virtual void OnSelectTab(FsGuiTabControl *tab,int newTab);
};

/* } */
#endif
