/* ////////////////////////////////////////////////////////////

  File Name: ysshellextdrawbuffer.h
  Copyright (c) 2014 Soji Yamakawa.  All rights reserved.
  http://www.ysflight.com

//////////////////////////////////////////////////////////// */

#ifndef YSSHELLEXTDRAWBUFFER_IS_INCLUDED
#define YSSHELLEXTDRAWBUFFER_IS_INCLUDED
/* { */

#include <ysglbuffer.h>
#include "ysshellext.h"

class YsShellExtDrawingBuffer
{
	class RemakePolygonBufferTask;
	class RemakeSelectedPolygonBufferTask;
	class RemakeSelectedFaceGroupBufferTask;

public:
	YSBOOL viewPortEnabled;
	YsVec3 viewPort[2];

	YSBOOL drawBackFaceInDifferentColor;
	YsColor backFaceColor;

	// Memo: I wish point-sprite feature worked as advertised.  According to my experiments, 
	//       point-sprite implementations in the various vendor drivers are very unreliable.  
	//       Therefore, each vertex is drawn as a triangles.  Therefore, one real vertex 
	//       takes three OpenGL vertices and an index is stored per triangle.

	YsGLVertexBuffer normalVertexPosBuffer;
	YsGLPixOffsetBuffer normalVertexPixOffsetBuffer;
	YsArray <int> normalVertexIdxBuffer;

	YsGLVertexBuffer roundVertexPosBuffer;
	YsGLPixOffsetBuffer roundVertexPixOffsetBuffer;
	YsArray <int> roundVertexIdxBuffer;

	YsGLVertexBuffer selectedVertexPosBuffer;
	YsGLPixOffsetBuffer selectedVertexPixOffsetBuffer;
	YsGLColorBuffer selectedVertexColBuffer;
	YsArray <int> selectedVertexIdxBuffer;

	YsGLVertexBuffer selectedVertexLineBuffer;
	YsGLColorBuffer selectedVertexLineColBuffer;

	YsGLVertexBuffer normalEdgePosBuffer;
	YsArray <int> normalEdgeIdxBuffer;
	YsGLVertexBuffer shrunkEdgePosBuffer;
	YsArray <int> shrunkEdgeIdxBuffer;

	YsGLVertexBuffer solidShadedVtxBuffer;
	YsGLNormalBuffer solidShadedNomBuffer;
	YsGLColorBuffer solidShadedColBuffer;
	YsArray <int> solidShadedIdxBuffer;

	YsGLVertexBuffer solidUnshadedVtxBuffer;
	YsGLColorBuffer solidUnshadedColBuffer;
	YsArray <int> solidUnshadedIdxBuffer;

	YsGLVertexBuffer trspShadedVtxBuffer;
	YsGLNormalBuffer trspShadedNomBuffer;
	YsGLColorBuffer trspShadedColBuffer;
	YsArray <int> trspShadedIdxBuffer;

	YsGLVertexBuffer trspUnshadedVtxBuffer;
	YsGLColorBuffer trspUnshadedColBuffer;
	YsArray <int> trspUnshadedIdxBuffer;

	YsGLVertexBuffer backFaceVtxBuffer;
	YsGLColorBuffer backFaceColBuffer;
	YsArray <int> backFaceIdxBuffer;

	YsGLVertexBuffer lightVtxBuffer;
	YsGLColorBuffer lightColBuffer;
	YsGLPixOffsetBuffer lightPixOffsetBuffer;

	YsGLVertexBuffer selectedPolygonPosBuffer;
	YsGLColorBuffer selectedPolygonColBuffer;
	YsArray <int> selectedPolygonIdxBuffer;

	YsGLVertexBuffer constEdgeVtxBuffer;
	YsArray <int> constEdgeIdxBuffer;

	YsGLVertexBuffer selectedConstEdgeVtxBuffer;
	YsArray <int> selectedConstEdgeIdxBuffer;

	YsGLVertexBuffer selectedConstEdgeArrowHeadVtxBuffer;

	YsGLVertexBuffer selectedFaceGroupVtxBuffer;
	YsArray <int> selectedFaceGroupIdxBuffer;


public:
	YsShellExtDrawingBuffer();
	void EnableViewPort(void);
	void DisableViewPort(void);
	void SetViewPort(const YsVec3 viewPort[2]);

	void RemakeNormalVertexBuffer(const class YsShellExt &shl,const double normalVertexMarkerRadius,const double roundVertexMarkerRadius);
	void RemakeSelectedVertexBuffer(const class YsShellExt &shl,const double selectedVertexMarkerRadius,YSSIZE_T nSel,const YsShellVertexHandle selVtHd[]);

	template <const int N>
	void RemakeSelectedVertexBuffer(const class YsShellExt &shl,const double selectedVertexMarkerRadius,const YsArray <YsShellVertexHandle,N> &selVtHd);
	void RemakePolygonBuffer(const class YsShellExt &shl,const double polygonShrinkRatioForPicking); // Cannot take const because it re-calculates vertex-normal

	void RemakeSelectedPolygonBuffer(const class YsShellExt &shl,const double polygonShrinkRatio,YSSIZE_T nSel,const YsShellPolygonHandle selPlHd[]);
	template <const int N>
	void RemakeSelectedPolygonBuffer(const class YsShellExt &shl,const double polygonShrinkRatio,const YsArray <YsShellPolygonHandle,N> &selPlHd);

	void RemakeConstEdgeBuffer(const class YsShellExt &shl,YSSIZE_T nSel,const YsShellExt::ConstEdgeHandle ceHdArray[]);
	template <const int N>
	void RemakeConstEdgeBuffer(const class YsShellExt &shl,const YsArray <YsShellExt::ConstEdgeHandle,N> &selCeHd);

	void RemakeSelectedConstEdgeBuffer(const class YsShellExt &shl,YSSIZE_T nSel,const YsShellExt::ConstEdgeHandle ceHdArray[]);
	template <const int N>
	void RemakeSelectedConstEdgeBuffer(const class YsShellExt &shl,const YsArray <YsShellExt::ConstEdgeHandle,N> &selCeHd);

	void RemakeSelectedFaceGroupBuffer(const class YsShellExt &shl,YSSIZE_T nFg,const YsShellExt::FaceGroupHandle fgHdArray[]);
	template <const int N>
	void RemakeSelectedFaceGroupBuffer(const class YsShellExt &shl,const YsArray <YsShellExt::FaceGroupHandle,N> &fgHdArray);

private:
	void MakeVertexNormalCache(YsHashTable <YsVec3> &vtxNomHash,const YsShellExt &shl);
	static YsColor ColorGradation(const double t);
	static YsColor ColorGradation(int idx,int n);
};

template <const int N>
void YsShellExtDrawingBuffer::RemakeSelectedVertexBuffer(const class YsShellExt &shl,const double selectedVertexMarkerRadius,const YsArray <YsShellVertexHandle,N> &selVtHd)
{
	RemakeSelectedVertexBuffer(shl,selectedVertexMarkerRadius,selVtHd.GetN(),selVtHd);
}

template <const int N>
void YsShellExtDrawingBuffer::RemakeSelectedPolygonBuffer(const class YsShellExt &shl,const double polygonShrinkRatio,const YsArray <YsShellPolygonHandle,N> &selPlHd)
{
	RemakeSelectedPolygonBuffer(shl,polygonShrinkRatio,selPlHd.GetN(),selPlHd);
}

template <const int N>
void YsShellExtDrawingBuffer::RemakeConstEdgeBuffer(const class YsShellExt &shl,const YsArray <YsShellExt::ConstEdgeHandle,N> &selCeHd)
{
	RemakeConstEdgeBuffer(shl,selCeHd.GetN(),selCeHd);
}

template <const int N>
void YsShellExtDrawingBuffer::RemakeSelectedConstEdgeBuffer(const class YsShellExt &shl,const YsArray <YsShellExt::ConstEdgeHandle,N> &selCeHd)
{
	RemakeSelectedConstEdgeBuffer(shl,selCeHd.GetN(),selCeHd);
}

template <const int N>
void YsShellExtDrawingBuffer::RemakeSelectedFaceGroupBuffer(const class YsShellExt &shl,const YsArray <YsShellExt::FaceGroupHandle,N> &fgHdArray)
{
	RemakeSelectedFaceGroupBuffer(shl,fgHdArray.GetN(),fgHdArray);
}

/* } */
#endif
