/* ////////////////////////////////////////////////////////////

  File Name: ysshellext_duplicateutil.h
  Copyright (c) 2014 Soji Yamakawa.  All rights reserved.
  http://www.ysflight.com

//////////////////////////////////////////////////////////// */

#ifndef YSSHELLEXT_DUPLICATEUTIL_IS_INCLUDED
#define YSSHELLEXT_DUPLICATEUTIL_IS_INCLUDED
/* { */

#include "ysshellext_mappingutil.h"

class YsShellExt_DuplicateUtil
{
protected:
	const class YsShellExt *srcShl;

private:
	YsShellExt_Mapping <YsShellVertexHandle,YSSIZE_T> vtHdMap;
	YsShellExt_Mapping <YsShellPolygonHandle,YSSIZE_T> plHdMap;
	YsShellExt_Mapping <YsShellExt::ConstEdgeHandle,YSSIZE_T> ceHdMap;
	YsShellExt_Mapping <YsShellExt::FaceGroupHandle,YSSIZE_T> fgHdMap;
	YsShellExt_Mapping <YsShellExt::VolumeHandle,YSSIZE_T> vlHdMap;

public:
	class Vertex
	{
	public:
		YsShellVertexHandle srcVtHd,newVtHd;
		YsVec3 pos;
		YsShellExt::VertexAttrib attrib;
		YSBOOL derived;
	};
	class Polygon
	{
	public:
		YsShellPolygonHandle srcPlHd,newPlHd;

		YsArray <YSSIZE_T> vertexArray;
		YsShellExt::PolygonAttrib attrib;
		YsColor col;
		YsVec3 nom;
		YSBOOL derived;
	};
	class ConstEdge
	{
	public:
		YsShellExt::ConstEdgeHandle srcCeHd,newCeHd;
		YSBOOL isLoop;
		YsArray <YSSIZE_T> vertexArray;
		YsShellExt::ConstEdge attrib;
		YSBOOL derived;
	};
	class FaceGroup
	{
	public:
		YsShellExt::FaceGroupHandle srcFgHd,newFgHd;
		YsArray <YSSIZE_T> polygonArray;
		YsShellExt::FaceGroup attrib;
		YSBOOL derived;
	};
	class Volume
	{
	public:
		YsShellExt::VolumeHandle srcVlHd,newVlHd;
		YsArray <YSSIZE_T> faceGroupArray;
		YsShellExt::Volume attrib;
		YSBOOL derived;
	};

	YsArray <Vertex> vertexArray;
	YsArray <Polygon> polygonArray;
	YsArray <ConstEdge> constEdgeArray;
	YsArray <FaceGroup> faceGroupArray;
	YsArray <Volume> volumeArray;

	YsShellExt_DuplicateUtil();

	YsShellExt_DuplicateUtil(const YsShellExt_DuplicateUtil &incoming);
	YsShellExt_DuplicateUtil &operator=(const YsShellExt_DuplicateUtil &incoming);
	YsShellExt_DuplicateUtil &CopyFrom(const YsShellExt_DuplicateUtil &incoming);

	YsShellExt_DuplicateUtil(YsShellExt_DuplicateUtil &&incoming);
	YsShellExt_DuplicateUtil &operator=(YsShellExt_DuplicateUtil &&incoming);
	YsShellExt_DuplicateUtil &MoveFrom(YsShellExt_DuplicateUtil &incoming);

	void CleanUp(void);

	// Step 1
	void SetShell(const YsShellExt &shl);

	// Step 2
	void AddVertex(YSSIZE_T n,const YsShellVertexHandle hdArray[]);
	template <class T>
	void AddVertex(const T &hdStore);
private:
	YSSIZE_T AddVertex(YsShellVertexHandle vtHd,YSBOOL derived);

public:
	// Step 3
	void AddPolygon(YSSIZE_T n,const YsShellPolygonHandle hdArray[]);
	template <class T>
	void AddPolygon(const T &hdStore);
private:
	YSSIZE_T AddPolygon(YsShellPolygonHandle plHd,YSBOOL derived);

public:
	// Step 4
	void AddConstEdge(YSSIZE_T n,const YsShellExt::ConstEdgeHandle hdArray[]);
	template <class T>
	void AddConstEdge(const T &hdStore);
private:
	YSSIZE_T AddConstEdge(YsShellExt::ConstEdgeHandle ceHd,YSBOOL derived);

public:
	// Step 5
	void AddFaceGroup(YSSIZE_T n,const YsShellExt::FaceGroupHandle hdArray[]);
	template <class T>
	void AddFaceGroup(const T &hdStore);
private:
	YSSIZE_T AddFaceGroup(YsShellExt::FaceGroupHandle fgHd,YSBOOL derived);

public:
	// Step 6
	void AddVolume(YSSIZE_T n,const YsShellExt::VolumeHandle hdArray[]);
	template <class T>
	void AddVolume(const T &hdStore);
private:
	YSSIZE_T AddVolume(YsShellExt::VolumeHandle vlHd,YSBOOL derived);


public:
	YsArray <YsShellVertexHandle> GetNonDerivedNewVertex(void) const;
	YsArray <YsShellPolygonHandle> GetNonDerivedNewPolygon(void) const;
	YsArray <YsShellExt::ConstEdgeHandle> GetNonDerivedNewConstEdge(void) const;
	YsArray <YsShellExt::FaceGroupHandle> GetNonDerivedNewFaceGroup(void) const;
	YsArray <YsShellExt::VolumeHandle> GetNonDerivedNewVolume(void) const;
};


template <class T>
void YsShellExt_DuplicateUtil::AddVertex(const T &hdStore)
{
	for(auto vtHd : hdStore)
	{
		AddVertex(vtHd,YSFALSE);
	}
}

template <class T>
void YsShellExt_DuplicateUtil::AddPolygon(const T &hdStore)
{
	for(auto plHd : hdStore)
	{
		AddPolygon(plHd,YSFALSE);
	}
}

template <class T>
void YsShellExt_DuplicateUtil::AddConstEdge(const T &hdStore)
{
	for(auto ceHd : hdStore)
	{
		AddConstEdge(ceHd,YSFALSE);
	}
}

template <class T>
void YsShellExt_DuplicateUtil::AddFaceGroup(const T &hdStore)
{
	for(auto iter=hdStore.begin(); iter!=hdStore.end(); ++iter)
	{
		YsShellExt::FaceGroupHandle fgHd=*iter;
		AddFaceGroup(fgHd,YSFALSE);
	}
}

template <class T>
void YsShellExt_DuplicateUtil::AddVolume(const T &hdStore)
{
	for(auto vlHd : hdStore)
	{
		AddVolume(vlHd,YSFALSE);
	}
}



/* } */
#endif
