/* ////////////////////////////////////////////////////////////

  File Name: ysshellext_localop.cpp
  Copyright (c) 2014 Soji Yamakawa.  All rights reserved.
  http://www.ysflight.com

//////////////////////////////////////////////////////////// */

#include <stdio.h>
#include "ysshellext_localop.h"



void YsShell_LocalOperation::FaceGroupSplit::CleanUp(void)
{
	fgHd=NULL;
	fgPlHdArray.CleanUp();
}



////////////////////////////////////////////////////////////

YsShell_SwapInfo::YsShell_SwapInfo()
{
	CleanUp();
}

void YsShell_SwapInfo::CleanUp(void)
{
	orgDiagonal[0]=NULL;
	orgDiagonal[1]=NULL;
	newDiagonal[0]=NULL;
	newDiagonal[1]=NULL;
	quadVtHd[0]=NULL;
	quadVtHd[1]=NULL;
	quadVtHd[2]=NULL;
	quadVtHd[3]=NULL;
	triPlHd[0]=NULL;
	triPlHd[1]=NULL;

	newTriVtHd[0][0]=NULL;
	newTriVtHd[0][1]=NULL;
	newTriVtHd[0][2]=NULL;
	newTriVtHd[1][0]=NULL;
	newTriVtHd[1][1]=NULL;
	newTriVtHd[1][2]=NULL;
}

YSRESULT YsShell_SwapInfo::MakeInfo(const YsShell &shl,const YsShellVertexHandle edVtHd[2])
{
	return MakeInfo(shl,edVtHd[0],edVtHd[1]);
}

YSRESULT YsShell_SwapInfo::MakeInfo(const YsShell &shl,const YsShellVertexHandle edVtHd0,const YsShellVertexHandle edVtHd1)
{
	const YsShellSearchTable *search=shl.GetSearchTable();
	if(NULL!=search)
	{
		int nEdPl;
		const YsShellPolygonHandle *edPlHd;
		if(YSOK==search->FindPolygonListByEdge(nEdPl,edPlHd,shl,edVtHd0,edVtHd1) && 
		   2==nEdPl &&
		   3==shl.GetNumVertexOfPolygon(edPlHd[0]) &&
		   3==shl.GetNumVertexOfPolygon(edPlHd[1]) &&
		   edPlHd[0]!=edPlHd[1])
		{
			YsArray <YsShellVertexHandle,4> plVtHd0;
			shl.GetVertexListOfPolygon(plVtHd0,edPlHd[0]);

			for(int i=0; i<3 && YSTRUE!=YsSameEdge(plVtHd0[0],plVtHd0[2],edVtHd0,edVtHd1); ++i)
			{
				YsShellVertexHandle swp=plVtHd0[0];
				plVtHd0[0]=plVtHd0[1];
				plVtHd0[1]=plVtHd0[2];
				plVtHd0[2]=swp;
			}

			YsArray <YsShellVertexHandle,4> plVtHd1;
			shl.GetVertexListOfPolygon(plVtHd1,edPlHd[1]);
			for(int i=0; i<3; ++i)
			{
				if(plVtHd1[i]!=edVtHd0 && plVtHd1[i]!=edVtHd1)
				{
					quadVtHd[0]=plVtHd0[0];
					quadVtHd[1]=plVtHd0[1];
					quadVtHd[2]=plVtHd0[2];
					quadVtHd[3]=plVtHd1[i];

					orgDiagonal[0]=edVtHd0;
					orgDiagonal[1]=edVtHd1;

					newDiagonal[0]=quadVtHd[1];
					newDiagonal[1]=quadVtHd[3];

					triPlHd[0]=edPlHd[0];
					triPlHd[1]=edPlHd[1];

					newTriVtHd[0][0]=quadVtHd[1];
					newTriVtHd[0][1]=quadVtHd[2];
					newTriVtHd[0][2]=quadVtHd[3];

					newTriVtHd[1][0]=quadVtHd[3];
					newTriVtHd[1][1]=quadVtHd[0];
					newTriVtHd[1][2]=quadVtHd[1];

					return YSOK;
				}
			}
		}
	}
	return YSERR;
}

YSRESULT YsShell_SwapInfo::MakeInfo(const YsShell &shl,const YsShellPolygonHandle edPlHd[2])
{
	return MakeInfo(shl,edPlHd[0],edPlHd[1]);
}

YSRESULT YsShell_SwapInfo::MakeInfo(const YsShell &shl,const YsShellPolygonHandle edPlHd0,const YsShellPolygonHandle edPlHd1)
{
	if(edPlHd0!=edPlHd1 && 3==shl.GetNumVertexOfPolygon(edPlHd0) && 3==shl.GetNumVertexOfPolygon(edPlHd1))
	{
		YsArray <YsShellVertexHandle,4> plVtHd[2];
		shl.GetVertexListOfPolygon(plVtHd[0],edPlHd0);
		shl.GetVertexListOfPolygon(plVtHd[1],edPlHd1);

		for(int i=0; i<3; ++i)
		{
			for(int j=0; j<3; ++j)
			{
				if(YSTRUE==YsSameEdge(plVtHd[0][i],plVtHd[0].GetCyclic(i+1),plVtHd[1][j],plVtHd[1].GetCyclic(j+1)))
				{
					return MakeInfo(shl,plVtHd[0][i],plVtHd[0].GetCyclic(i+1));
				}
			}
		}
	}
	return YSERR;
}

////////////////////////////////////////////////////////////

YsShell_MergeInfo::YsShell_MergeInfo()
{
	CleanUp();
}

void YsShell_MergeInfo::CleanUp(void)
{
	plHd[0]=NULL;
	plHd[1]=NULL;
	newPlVtHd.CleanUp();
	duplicateWarning=YSFALSE;
}

YSRESULT YsShell_MergeInfo::MakeInfo(const YsShell &shl,const YsShellPolygonHandle plHd[2])
{
	return MakeInfo(shl,plHd[0],plHd[1]);
}

YSRESULT YsShell_MergeInfo::MakeInfo(const YsShell &shl,const YsShellPolygonHandle plHd0,const YsShellPolygonHandle plHd1)
{
	CleanUp();

	YsArray <YsShellVertexHandle> plVtHd[2];
	shl.GetVertexListOfPolygon(plVtHd[0],plHd0);
	shl.GetVertexListOfPolygon(plVtHd[1],plHd1);

printf("%s %d\n",__FUNCTION__,__LINE__);
	for(int i=0; i<2; ++i)
	{
		printf("[%d]",i);
		for(int j=0; j<plVtHd[i].GetN(); ++j)
		{
			printf(" %d",shl.GetSearchKey(plVtHd[i][j]));
		}
		printf("\n");
	}

	// Find first shared edge
	int sharedEdIdx[2]={-1,-1};
	for(int i=0; i<plVtHd[0].GetN(); ++i)
	{
		for(int j=0; j<plVtHd[1].GetN(); ++j)
		{
			if(plVtHd[0][i]==plVtHd[1][j] && plVtHd[0].GetCyclic(i+1)==plVtHd[1].GetCyclic(j+1))
			{
				sharedEdIdx[0]=i;
				sharedEdIdx[1]=j;
			}
			else if(plVtHd[0][i]==plVtHd[1].GetCyclic(j+1) && plVtHd[0].GetCyclic(i+1)==plVtHd[1][j])
			{
				sharedEdIdx[0]=i;
				plVtHd[1].Invert();
				sharedEdIdx[1]=plVtHd[1].GetN()-1-(j+1)%plVtHd[1].GetN();
			}
		}
	}

printf("%s %d\n",__FUNCTION__,__LINE__);
	// Validate
	if(0>sharedEdIdx[0] || 0>sharedEdIdx[1])
	{
		return YSERR;
	}
printf("%s %d\n",__FUNCTION__,__LINE__);
	if(YSTRUE!=YsSameEdge(plVtHd[0][sharedEdIdx[0]],plVtHd[0].GetCyclic(1+sharedEdIdx[0]),
	                      plVtHd[1][sharedEdIdx[1]],plVtHd[1].GetCyclic(1+sharedEdIdx[1])))
	{
		YsPrintf("%s\n",__FUNCTION__);
		YsPrintf("  Internal error.  Shared edge identification failed.\n");
		return YSERR;
	}

printf("%d %d\n",sharedEdIdx[0],sharedEdIdx[1]);
printf("%s %d\n",__FUNCTION__,__LINE__);
	// Rotate
	{
		YsArray <YsShellVertexHandle,16> A,B;
		if(0!=sharedEdIdx[0])
		{
			A.Set(sharedEdIdx[0],plVtHd[0]);
			B.Set(plVtHd[0].GetN()-sharedEdIdx[0],plVtHd[0]+sharedEdIdx[0]);
			plVtHd[0]=B;
			plVtHd[0].Append(A);
		}

		if(0!=sharedEdIdx[1])
		{
			A.Set(sharedEdIdx[1],plVtHd[1]);
			B.Set(plVtHd[1].GetN()-sharedEdIdx[1],plVtHd[1]+sharedEdIdx[1]);
			plVtHd[1]=B;
			plVtHd[1].Append(A);
		}
	}

printf("%s %d\n",__FUNCTION__,__LINE__);
	// Trim mid shared sequence
	while(3<=plVtHd[0].GetN() &&
	      3<=plVtHd[1].GetN() &&
	      plVtHd[0][0]==plVtHd[1][0] &&
	      plVtHd[0][1]==plVtHd[1][1] &&
	      plVtHd[0][2]==plVtHd[1][2])
	{
		plVtHd[0].Delete(1);
		plVtHd[1].Delete(1);
	}
	while(3<=plVtHd[0].GetN() &&
	      3<=plVtHd[1].GetN() &&
	      plVtHd[0].GetEnd()==plVtHd[1].GetEnd() &&
	      plVtHd[0][0]==plVtHd[1][0] &&
	      plVtHd[0][1]==plVtHd[1][1])
	{
		YsShellVertexHandle swp=plVtHd[0].GetEnd();
		plVtHd[0][0]=swp;
		plVtHd[0].DeleteLast();

		swp=plVtHd[1].GetEnd();
		plVtHd[1][0]=swp;
		plVtHd[1].DeleteLast();
	}

printf("%d %d\n",plVtHd[0].GetN(),plVtHd[1].GetN());
	for(int i=0; i<2; ++i)
	{
		printf("[%d]",i);
		for(int j=0; j<plVtHd[i].GetN(); ++j)
		{
			printf(" %d",shl.GetSearchKey(plVtHd[i][j]));
		}
		printf("\n");
	}

printf("%s %d\n",__FUNCTION__,__LINE__);
	// Finally assemble merged polygon
	if(2<=plVtHd[0].GetN() && 
	   2<=plVtHd[1].GetN() &&
	   YSTRUE==YsSameEdge(plVtHd[0][0],plVtHd[0][1],plVtHd[1][0],plVtHd[1][1]))
	{
printf("%s %d\n",__FUNCTION__,__LINE__);
		YsShellVertexHandle swp=plVtHd[0][0];
		plVtHd[0].Append(swp);
		plVtHd[0].Delete(0);

		swp=plVtHd[1][0];
		plVtHd[1].Append(swp);
		plVtHd[1].Delete(0);

		newPlVtHd=plVtHd[0];
		plVtHd[1].Invert();
		newPlVtHd.Append(plVtHd[1].GetN()-2,plVtHd[1]+1);

		for(int i=0; i<newPlVtHd.GetN(); ++i)
		{
			for(int j=i+1; j<newPlVtHd.GetN(); ++j)
			{
				if(newPlVtHd[i]==newPlVtHd[j])
				{
					duplicateWarning=YSTRUE;
				}
			}
		}

		if(3<=newPlVtHd.GetN())
		{
			return YSOK;
		}
	}
	return YSERR;
}

////////////////////////////////////////////////////////////

YsShell_CollapseInfo::YsShell_CollapseInfo()
{
	CleanUp();
}

void YsShell_CollapseInfo::CleanUp(void)
{
	newPlg.Set(0,NULL);
	newConstEdge.Set(0,NULL);
	plHdToDel.Set(0,NULL);
	ceHdToDel.Set(0,NULL);
	vtHdToDel=NULL;
	vtHdToSurvive=NULL;
}

YSRESULT YsShell_CollapseInfo::MakeInfo(const YsShellExt &shl,const YsShellVertexHandle edVtHd[2])
{
	return MakeInfo(shl,edVtHd[0],edVtHd[1]);
}

YSRESULT YsShell_CollapseInfo::MakeInfo(const YsShellExt &shl,const YsShellVertexHandle vtHdToDel,const YsShellVertexHandle vtHdToSurvive)
{
	CleanUp();

	YsArray <YsShellPolygonHandle> vtPlHd;
	if(YSOK==shl.FindPolygonFromVertex(vtPlHd,vtHdToDel))
	{
		for(YSSIZE_T plIdx=0; plIdx<vtPlHd.GetN(); ++plIdx)
		{
			YsArray <YsShellVertexHandle,4> plVtHd;
			shl.GetVertexListOfPolygon(plVtHd,vtPlHd[plIdx]);

			for(int i=plVtHd.GetN()-1; 0<=i; --i)
			{
				if(plVtHd[i]==vtHdToDel)
				{
					plVtHd[i]=vtHdToSurvive;
				}

				if(plVtHd[i]==plVtHd.GetCyclic(i+1))
				{
					plVtHd.Delete(i);
				}
			}

			if(0<plVtHd.GetN() && plVtHd[0]==plVtHd.Last())
			{
				plVtHd.DeleteLast();
			}

			if(3<=plVtHd.GetN())
			{
				newPlg.Increment();
				newPlg.GetEnd().plHd=vtPlHd[plIdx];
				newPlg.GetEnd().plVtHd=plVtHd;
			}
			else
			{
				plHdToDel.Append(vtPlHd[plIdx]);
			}
		}
	}

	YsArray <YsShellExt::ConstEdgeHandle> vtCeHd;
	if(YSOK==shl.FindConstEdgeFromVertex(vtCeHd,vtHdToDel))
	{
		for(YSSIZE_T ceIdx=0; ceIdx<vtCeHd.GetN(); ++ceIdx)
		{
			YsArray <YsShellVertexHandle,4> ceVtHd;
			YSBOOL isLoop;
			shl.GetConstEdge(ceVtHd,isLoop,vtCeHd[ceIdx]);

			if(YSTRUE!=isLoop)
			{
				for(int i=ceVtHd.GetN()-1; 0<=i; --i)
				{
					if(ceVtHd[i]==vtHdToDel)
					{
						ceVtHd[i]=vtHdToSurvive;
					}

					if(ceVtHd[i]==ceVtHd.GetCyclic(i+1))
					{
						ceVtHd.Delete(i);
					}
				}

				if(2<=ceVtHd.GetN())
				{
					newConstEdge.Increment();
					newConstEdge.GetEnd().ceHd=vtCeHd[ceIdx];
					newConstEdge.GetEnd().ceVtHd=ceVtHd;
					newConstEdge.GetEnd().isLoop=isLoop;
				}
				else
				{
					ceHdToDel.Append(vtCeHd[ceIdx]);
				}
			}
			else
			{
				for(int i=ceVtHd.GetN()-1; 0<=i; --i)
				{
					if(ceVtHd[i]==vtHdToDel)
					{
						ceVtHd[i]=vtHdToSurvive;
					}

					if(ceVtHd[i]==ceVtHd.GetCyclic(i+1))
					{
						ceVtHd.Delete(i);
					}
				}

				if(3<=ceVtHd.GetN())
				{
					newConstEdge.Increment();
					newConstEdge.GetEnd().ceHd=vtCeHd[ceIdx];
					newConstEdge.GetEnd().ceVtHd=ceVtHd;
					newConstEdge.GetEnd().isLoop=isLoop;
				}
				else if(2==ceVtHd.GetN())
				{
					newConstEdge.Increment();
					newConstEdge.GetEnd().ceHd=vtCeHd[ceIdx];
					newConstEdge.GetEnd().ceVtHd=ceVtHd;
					newConstEdge.GetEnd().isLoop=YSFALSE;
				}
				else
				{
					ceHdToDel.Append(vtCeHd[ceIdx]);
				}
			}
		}
	}

	this->vtHdToSurvive=vtHdToSurvive;
	this->vtHdToDel=vtHdToDel;

	return YSOK;
}

