/* ////////////////////////////////////////////////////////////

  File Name: ysshellext_trackingutil.h
  Copyright (c) 2014 Soji Yamakawa.  All rights reserved.
  http://www.ysflight.com

//////////////////////////////////////////////////////////// */

#ifndef YSSHELLEXT_TRACKINGUTIL_IS_INCLUDED
#define YSSHELLEXT_TRACKINGUTIL_IS_INCLUDED
/* { */

#include <ysclass.h>
#include <ysshellext.h>

class YsShellExt_TrackingUtil
{
public:
	class SearchCondition
	{
	public:
		// /*! Combination of flags. */
	    // unsigned int searchFlag;

	    /*! If withinThesePlHd is non-NULL, this function will only search within polygons included in withinThesePlHd. */
	    const YsShellPolygonStore *withinThesePlHd;

		/*! If excludeThesePlHd is non-NULL, this function will not go through the polygons included in excludeThesePlHd. */
	    const YsShellPolygonStore *excludeThesePlHd;

		/*! If onlyInThisFgHd is non-NULL, this function will not search outside this face group. */
		YsShellExt::FaceGroupHandle onlyInThisFgHd;

		SearchCondition();

		void Initialize(void);

		virtual YSRESULT TestPolygon(const YsShellExt &shl,YsShellPolygonHandle plHd) const;
		virtual YSRESULT TestEdge(const YsShellExt &shl,YsShellVertexHandle edVtHd0,YsShellVertexHandle edVtHd1) const;
	};

	/*! This function tracks vertices based on the connection.  
	   It starts from fromVtHd toward toVtHd and stops when it can no longer find a unique path.
	   If the tracking makes a loop, the first and the last vertices of the returned array will be the same. */
	static YsArray <YsShellVertexHandle> TrackVertexConnection(
		const YsShell &shl,
		YsShellVertexHandle fromVtHd,YsShellVertexHandle toVtHd,
		const YsFixedLengthToMultiHashTable <YsShellVertexHandle,1,2> &connection);

	/*! This function finds and returns a set of polygons on one side of a sequence of vertices. 
	    Every segment in the vertex sequence must at least be used by a polygon.
	    If it cannot find a set of polygons incident to the vertex sequence and
	    on the same side as the given polygons it returns an empty array.  */
	static YsArray <YsShellPolygonHandle> TrackPolygonOneSideOfVertexSequence(
	    const YsShellExt &shl,
	    YSSIZE_T nVt,const YsShellVertexHandle vtHdArray[],
		YSSIZE_T nFlagPl,const YsShellPolygonHandle flagPlHdArray[]);

	/*! Templated version of TrackPolygonOneSideOfVertexSequence */
	template <const int N,const int M>
	static YsArray <YsShellPolygonHandle> TrackPolygonOneSideOfVertexSequence(const YsShellExt &shl,const YsArray <YsShellVertexHandle,N> &vtHdArray,const YsArray <YsShellPolygonHandle,M> &plHdArray);

	/*! This function finds all edge-connected polygons starting from seedPlHd.
	    If searchCondition is non-NULL, the search space will be limited by the condition described in searchCondition.
	*/
	static YsArray <YsShellPolygonHandle> MakeEdgeConnectedPolygonGroup(
	    const YsShellExt &shl,YsShellPolygonHandle seedPlHd,SearchCondition *searchCondition);

	/*! This function returns a sequence of vertices by connecting specified constraint edges. */
	static YsArray <YsShellVertexHandle> MakeVertexSequenceFromMultiConstEdge(
	    const YsShellExt &shl,YSSIZE_T nCe,const YsShellExt::ConstEdgeHandle ceHdArray[]);

	/*! This function returns a sequence of vertices by connecting specified constraint edges. */
	template <const int N>
	static YsArray <YsShellVertexHandle> MakeVertexSequenceFromMultiConstEdge(
	    const YsShellExt &shl,const YsArray <YsShellExt::ConstEdgeHandle,N> &ceHdArray);
};

template <const int N,const int M>
/*static*/ YsArray <YsShellPolygonHandle> YsShellExt_TrackingUtil::TrackPolygonOneSideOfVertexSequence(const YsShellExt &shl,const YsArray <YsShellVertexHandle,N> &vtHdArray,const YsArray <YsShellPolygonHandle,M> &plHdArray)
{
	return TrackPolygonOneSideOfVertexSequence(shl,vtHdArray.GetN(),vtHdArray,plHdArray.GetN(),plHdArray);
}

template <const int N>
/*static*/ YsArray <YsShellVertexHandle> YsShellExt_TrackingUtil::MakeVertexSequenceFromMultiConstEdge(
    const YsShellExt &shl,const YsArray <YsShellExt::ConstEdgeHandle,N> &ceHdArray)
{
	return MakeVertexSequenceFromMultiConstEdge(shl,ceHdArray.GetN(),ceHdArray);
}


/* } */
#endif
