/* ////////////////////////////////////////////////////////////

  File Name: ysshellextedit_duplicateutil.cpp
  Copyright (c) 2014 Soji Yamakawa.  All rights reserved.
  http://www.ysflight.com

//////////////////////////////////////////////////////////// */

#include "ysshellextedit_duplicateutil.h"
#include <ysshellextedit.h>


YSRESULT YsShellExtEdit_DuplicateUtil::Paste(class YsShellExtEdit &dstShl)
{
	YsShellExtEdit::StopIncUndo incUndo(&dstShl);

	for(auto &vtx : vertexArray)
	{
		auto newPos=vtx.pos;
		vtx.newVtHd=dstShl.AddVertex(newPos);
	}

	for(auto &plg : polygonArray)
	{
		YsArray <YsShellVertexHandle,4> newPlVtHd;
		newPlVtHd.Set(plg.vertexArray.GetN(),NULL);
		for(YSSIZE_T idx=0; idx<plg.vertexArray.GetN(); ++idx)
		{
			auto vtIdx=plg.vertexArray[idx];
			auto &vtx=vertexArray[vtIdx];
			newPlVtHd[idx]=vtx.newVtHd;
		}

		auto newPlHd=dstShl.AddPolygon(newPlVtHd);
		dstShl.SetPolygonAttrib(newPlHd,plg.attrib);
		dstShl.SetPolygonColor(newPlHd,plg.col);

		dstShl.SetPolygonNormal(newPlHd,plg.nom);
		plg.newPlHd=newPlHd;
	}

	PasteConstEdge(dstShl);
	PasteFaceGroup(dstShl);

	return YSOK;
}


YSRESULT YsShellExtEdit_DuplicateUtil::MakeAxisymmetric(class YsShellExtEdit &shl,const int component,YSBOOL dontMakeCopyOnSymmPlane,YSBOOL deletePolygonOnSymmPlane)
{
	YsShellExtEdit::StopIncUndo incUndo(&shl);

	for(auto &vtx : vertexArray)
	{
		auto newPos=vtx.pos;
		if(YSTRUE==dontMakeCopyOnSymmPlane && YSTRUE==YsEqual(newPos[component],0.0) && (const YsShellExt *)&shl==srcShl)
		{
			vtx.newVtHd=vtx.srcVtHd;
		}
		else
		{
			newPos[component]=-newPos[component];
			vtx.newVtHd=shl.AddVertex(newPos);
		}
	}

	for(auto &plg : polygonArray)
	{
		YSBOOL outOfSymmPlane=YSFALSE;
		YsArray <YsShellVertexHandle,4> newPlVtHd;
		newPlVtHd.Set(plg.vertexArray.GetN(),NULL);
		for(YSSIZE_T idx=0; idx<plg.vertexArray.GetN(); ++idx)
		{
			auto vtIdx=plg.vertexArray[idx];
			auto &vtx=vertexArray[vtIdx];
			if(vtx.srcVtHd!=vtx.newVtHd)
			{
				outOfSymmPlane=YSTRUE;
			}
			newPlVtHd[newPlVtHd.GetN()-1-idx]=vtx.newVtHd;
		}

		if(YSTRUE==outOfSymmPlane || (const YsShellExt *)&shl!=srcShl)
		{
			auto newPlHd=shl.AddPolygon(newPlVtHd);
			shl.SetPolygonAttrib(newPlHd,plg.attrib);
			shl.SetPolygonColor(newPlHd,plg.col);

			YsVec3 newNom=plg.nom;
			newNom[component]=-newNom[component];
			shl.SetPolygonNormal(newPlHd,newNom);

			plg.newPlHd=newPlHd;
		}
		else
		{
			if(YSTRUE==deletePolygonOnSymmPlane)
			{
				shl.DeletePolygon(plg.srcPlHd);
			}
			plg.newPlHd=NULL;
		}
	}

	PasteConstEdgeForMirrorImage(shl);
	PasteFaceGroup(shl);

	return YSOK;
}



YSRESULT YsShellExtEdit_DuplicateUtil::MakeBySymmetryPlane(class YsShellExtEdit &shl,const YsPlane &pln,YSBOOL dontMakeCopyOnSymmPlane,YSBOOL deletePolygonOnSymmPlane)
{
	YsShellExtEdit::StopIncUndo incUndo(&shl);

	for(auto &vtx : vertexArray)
	{
		auto newPos=vtx.pos;
		if(YSTRUE==dontMakeCopyOnSymmPlane && YSTRUE==pln.CheckOnPlane(newPos) && (const YsShellExt *)&shl==srcShl)
		{
			vtx.newVtHd=vtx.srcVtHd;
		}
		else
		{
			const YsVec3 prj=pln.GetNormal()*(pln.GetNormal()*(newPos-pln.GetOrigin()));
			newPos=newPos-prj*2.0;
			vtx.newVtHd=shl.AddVertex(newPos);
		}
	}

	for(auto &plg : polygonArray)
	{
		YSBOOL outOfSymmPlane=YSFALSE;
		YsArray <YsShellVertexHandle,4> newPlVtHd;
		newPlVtHd.Set(plg.vertexArray.GetN(),NULL);
		for(YSSIZE_T idx=0; idx<plg.vertexArray.GetN(); ++idx)
		{
			auto vtIdx=plg.vertexArray[idx];
			auto &vtx=vertexArray[vtIdx];
			if(vtx.srcVtHd!=vtx.newVtHd)
			{
				outOfSymmPlane=YSTRUE;
			}
			newPlVtHd[newPlVtHd.GetN()-1-idx]=vtx.newVtHd;
		}

		if(YSTRUE==outOfSymmPlane || (const YsShellExt *)&shl!=srcShl)
		{
			auto newPlHd=shl.AddPolygon(newPlVtHd);
			shl.SetPolygonAttrib(newPlHd,plg.attrib);
			shl.SetPolygonColor(newPlHd,plg.col);

			const YsVec3 prj=pln.GetNormal()*(plg.nom*pln.GetNormal());
			const YsVec3 newNom=plg.nom-prj*2.0;
			shl.SetPolygonNormal(newPlHd,newNom);

			plg.newPlHd=newPlHd;
		}
		else
		{
			if(YSTRUE==deletePolygonOnSymmPlane)
			{
				shl.DeletePolygon(plg.srcPlHd);
			}
			plg.newPlHd=NULL;
		}
	}

	PasteConstEdgeForMirrorImage(shl);
	PasteFaceGroup(shl);

	return YSOK;
}



void YsShellExtEdit_DuplicateUtil::PasteConstEdgeForMirrorImage(class YsShellExtEdit &dstShl)
{
	for(auto &ce : constEdgeArray)
	{
		YSBOOL outOfSymmPlane=YSFALSE;
		YsArray <YsShellVertexHandle,4> newVtHd;
		newVtHd.Set(ce.vertexArray.GetN(),NULL);
		for(YSSIZE_T idx=0; idx<ce.vertexArray.GetN(); ++idx)
		{
			auto vtIdx=ce.vertexArray[idx];
			auto &vtx=vertexArray[vtIdx];
			if(vtx.srcVtHd!=vtx.newVtHd)
			{
				outOfSymmPlane=YSTRUE;
			}
			newVtHd[newVtHd.GetN()-1-idx]=vtx.newVtHd;
		}

		if(YSTRUE==outOfSymmPlane || (const YsShellExt *)&dstShl!=srcShl)
		{
			YsShellExt::ConstEdgeHandle newCeHd=dstShl.AddConstEdge(newVtHd,ce.isLoop);
			dstShl.SetConstEdgeAttrib(newCeHd,ce.attrib);
		}
	}
}

void YsShellExtEdit_DuplicateUtil::PasteConstEdge(class YsShellExtEdit &dstShl)
{
	for(auto &ce : constEdgeArray)
	{
		YsArray <YsShellVertexHandle,4> newVtHd;
		newVtHd.Set(ce.vertexArray.GetN(),NULL);
		for(YSSIZE_T idx=0; idx<ce.vertexArray.GetN(); ++idx)
		{
			auto vtIdx=ce.vertexArray[idx];
			auto &vtx=vertexArray[vtIdx];
			newVtHd[newVtHd.GetN()-1-idx]=vtx.newVtHd;
		}

		ce.newCeHd=dstShl.AddConstEdge(newVtHd,ce.isLoop);
		dstShl.SetConstEdgeAttrib(ce.newCeHd,ce.attrib);
	}
}

void YsShellExtEdit_DuplicateUtil::PasteFaceGroup(class YsShellExtEdit &dstShl)
{
	for(auto &fg : faceGroupArray)
	{
		YsArray <YsShellPolygonHandle> newPlHdArray;
		for(auto &newPlIdx : fg.polygonArray)
		{
			auto newPlHd=polygonArray[newPlIdx].newPlHd;
			if(NULL!=newPlHd)
			{
				newPlHdArray.Append(newPlHd);
			}
		}
		if(0<newPlHdArray.GetN())
		{
			fg.newFgHd=dstShl.AddFaceGroup(newPlHdArray);
			dstShl.SetFaceGroupAttrib(fg.newFgHd,fg.attrib);
		}
	}
}
