/* ////////////////////////////////////////////////////////////

  File Name: ysglbuffer.h
  Copyright (c) 2014 Soji Yamakawa.  All rights reserved.
  http://www.ysflight.com

//////////////////////////////////////////////////////////// */

#ifndef YSGLDRAWBUFFER_IS_INCLUDED
#define YSGLDRAWBUFFER_IS_INCLUDED
/* { */

#include <ysclass.h>

typedef float YSGLFLOAT;

class YsGLBuffer
{
public:
};

class YsGLVertexBuffer : public YsGLBuffer
{
public:
	YsArray <YSGLFLOAT> vtx;
	inline operator const YSGLFLOAT *() const;
	inline void CleanUp(void);
	inline void AddVertex(const YsVec3 &pos);
	template <class T>
	inline void AddVertex(T x,T y,T z);
	inline int GetNumVertex(void) const;
	inline void MoveFrom(YsGLVertexBuffer &from);

	YSRESULT AddArrowHeadTriangle(const YsVec3 &p0,const YsVec3 &p1,const double arrowSize);
	YSRESULT AddArrowHeadTriangle(const YsVec3 p[2],const double arrowSize);
};

inline YsGLVertexBuffer::operator const YSGLFLOAT *() const
{
	return vtx.GetArray();
}

inline void YsGLVertexBuffer::CleanUp(void)
{
	vtx.CleanUp();
}

inline void YsGLVertexBuffer::AddVertex(const YsVec3 &pos)
{
	vtx.Append(pos.xf());
	vtx.Append(pos.yf());
	vtx.Append(pos.zf());
}

template <class T>
inline void YsGLVertexBuffer::AddVertex(T x,T y,T z)
{
	vtx.Append((YSGLFLOAT)x);
	vtx.Append((YSGLFLOAT)y);
	vtx.Append((YSGLFLOAT)z);
}

inline int YsGLVertexBuffer::GetNumVertex(void) const
{
	return (int)vtx.GetN()/3;
}

inline void YsGLVertexBuffer::MoveFrom(YsGLVertexBuffer &from)
{
	vtx.Append(from.vtx);
	from.vtx.ClearDeep();
}



class YsGLPixOffsetBuffer : public YsGLBuffer
{
public:
	YsArray <YSGLFLOAT> pixOffset;

	inline operator const YSGLFLOAT *() const;
	inline void CleanUp(void);
	template <class T>
	inline void AddPixOffset(T x,T y);
	inline void MoveFrom(YsGLPixOffsetBuffer &from);
};

inline YsGLPixOffsetBuffer::operator const YSGLFLOAT *() const
{
	return pixOffset.GetArray();
}

inline void YsGLPixOffsetBuffer::CleanUp(void)
{
	pixOffset.CleanUp();
}

template <class T>
inline void YsGLPixOffsetBuffer::AddPixOffset(T x,T y)
{
	pixOffset.Append((YSGLFLOAT)x);
	pixOffset.Append((YSGLFLOAT)y);
}

inline void YsGLPixOffsetBuffer::MoveFrom(YsGLPixOffsetBuffer &from)
{
	pixOffset.Append(from.pixOffset);
	from.pixOffset.ClearDeep();
}



class YsGLColorBuffer : public YsGLBuffer
{
public:
	YsArray <YSGLFLOAT> col;
	inline operator const YSGLFLOAT *() const;
	inline void CleanUp(void);
	inline void AddColor(YsColor col);
	template <class T>
	inline void AddColor(YsColor col,T a);
	template <class T>
	inline void AddColor(T r,T g,T b,T a);
	template <class T>
	inline void AddColor(T r,T g,T b);
	inline void MoveFrom(YsGLColorBuffer &from);
};

inline YsGLColorBuffer::operator const YSGLFLOAT *() const
{
	return col.GetArray();
}

inline void YsGLColorBuffer::CleanUp(void)
{
	col.CleanUp();
}

inline void YsGLColorBuffer::AddColor(YsColor col)
{
	this->col.Append(col.Rf());
	this->col.Append(col.Gf());
	this->col.Append(col.Bf());
	this->col.Append(col.Af());
}

template <class T>
inline void YsGLColorBuffer::AddColor(YsColor col,T a)
{
	this->col.Append(col.Rf());
	this->col.Append(col.Gf());
	this->col.Append(col.Bf());
	this->col.Append(a);
}

template <class T>
inline void YsGLColorBuffer::AddColor(T r,T g,T b,T a)
{
	this->col.Append((YSGLFLOAT)r);
	this->col.Append((YSGLFLOAT)g);
	this->col.Append((YSGLFLOAT)b);
	this->col.Append((YSGLFLOAT)a);
}

template <class T>
inline void YsGLColorBuffer::AddColor(T r,T g,T b)
{
	this->col.Append((YSGLFLOAT)r);
	this->col.Append((YSGLFLOAT)g);
	this->col.Append((YSGLFLOAT)b);
	this->col.Append(1.0f);
}

inline void YsGLColorBuffer::MoveFrom(YsGLColorBuffer &from)
{
	col.Append(from.col);
	from.col.ClearDeep();
}



class YsGLNormalBuffer : public YsGLBuffer
{
public:
	YsArray <YSGLFLOAT> nom;
	inline operator const YSGLFLOAT *() const;
	inline void CleanUp(void);
	inline void AddNormal(const YsVec3 &nom);
	template <class T>
	inline void AddNormal(T x,T y,T z);
	inline void MoveFrom(YsGLNormalBuffer &from);
};

inline YsGLNormalBuffer::operator const YSGLFLOAT *() const
{
	return nom.GetArray();
}

inline void YsGLNormalBuffer::CleanUp(void)
{
	nom.CleanUp();
}

inline void YsGLNormalBuffer::AddNormal(const YsVec3 &nom)
{
	this->nom.Append(nom.xf());
	this->nom.Append(nom.yf());
	this->nom.Append(nom.zf());
}

template <class T>
inline void YsGLNormalBuffer::AddNormal(T x,T y,T z)
{
	this->nom.Append((YSGLFLOAT)x);
	this->nom.Append((YSGLFLOAT)y);
	this->nom.Append((YSGLFLOAT)z);
}

inline void YsGLNormalBuffer::MoveFrom(YsGLNormalBuffer &from)
{
	nom.Append(from.nom);
	from.nom.ClearDeep();
}



/* } */
#endif
